import $ from 'jquery';
import { SELECTOR, CLASS } from './constants';

export default class Overlay {
	constructor() {
		var focusableElems = {},
			current, this.onLast,
			this.prevState, elem;

		// Create document overlay
		this.elem = $('<div />', {
			id: 'qtip-overlay',
			html: '<div></div>',
			mousedown: function() { return false; }
		})
		.hide();

		// Make sure we can't focus anything outside the tooltip
		$(document.body).bind('focusin'+SELECTOR, () => {
			return this.stealFocus.apply(this, arguments);
		});

		// Apply keyboard "Escape key" close handler
		$(document).bind('keydown'+SELECTOR, (event) => {
			if(this.current && this.current.options.show.modal.escape && event.keyCode === 27) {
				this.current.hide(event);
			}
		});

		// Apply click handler for blur option
		this.elem.bind('click'+SELECTOR, (event) => {
			if(this.current && this.current.options.show.modal.blur) {
				this.current.hide(event);
			}
		});

		return this;
	}

	update(api) {
		// Update current API reference
		this.current = api;

		// Update focusable elements if enabled
		if(api.options.show.modal.stealfocus !== false) {
			focusableElems = api.tooltip.find('*').filter(() => {
				return this.focusable(this);
			});
		}
		else { focusableElems = []; }
	}

	toggle(api, state, duration) {
		var docBody = $(document.body),
			tooltip = api.tooltip,
			options = api.options.show.modal,
			effect = options.effect,
			type = state ? 'show': 'hide',
			visible = this.elem.is(':visible'),
			visibleModals = $(SELECTOR).filter(':visible:not(:animated)').not(tooltip),
			zindex;

		// Set active tooltip API reference
		this.update(api);

		// If the modal can steal the focus...
		// Blur the current item and focus anything in the modal we an
		if(state && options.stealfocus !== false) {
			this.focusInputs( $(':focus') );
		}

		// Toggle backdrop cursor style on show
		this.elem.toggleClass('blurs', options.blur);

		// Append to body on show
		if(state) {
			this.elem.appendTo(document.body);
		}

		// Prevent modal from conflicting with show.solo, and don't hide backdrop is other modals are visible
		if((this.elem.is(':animated') && visible === state && this.prevState !== false) || (!state && visibleModals.length)) {
			return this;
		}

		// Stop all animations
		this.elem.stop(true, false);

		// Use custom function if provided
		if($.isFunction(effect)) {
			effect.call(this.elem, state);
		}

		// If no effect type is supplied, use a simple toggle
		else if(effect === false) {
			this.elem[ type ]();
		}

		// Use basic fade function
		else {
			this.elem.fadeTo( parseInt(duration, 10) || 90, state ? 1 : 0, () => {
				if(!state) { this.elem.hide(); }
			});
		}

		// Reset position and detach from body on hide
		if(!state) {
			this.elem.queue((next) => {
				this.elem.css({ left: '', top: '' });
				if(!$(SELECTOR).length) {
					this.elem.detach();
				}
				next();
			});
		}

		// Cache the state
		this.prevState = state;

		// If the tooltip is destroyed, set reference to null
		if(this.current.destroyed) {
			this.current = null;
		}

		return this;
	}

	// Modified code from jQuery UI 1.10.0 source
	// http://code.jquery.com/ui/1.10.0/jquery-ui.js
	focusable(element) {
		// Use the defined focusable checker when possible
		if($.expr[':'].focusable) { return $.expr[':'].focusable; }

		var isTabIndexNotNaN = !isNaN($.attr(element, 'tabindex')),
			nodeName = element.nodeName && element.nodeName.toLowerCase(),
			map, mapName, img;

		if('area' === nodeName) {
			map = element.parentNode;
			mapName = map.name;
			if(!element.href || !mapName || map.nodeName.toLowerCase() !== 'map') {
				return false;
			}
			img = $('img[usemap=#' + mapName + ']')[0];
			return !!img && img.is(':visible');
		}
		return (/input|select|textarea|button|object/.test( nodeName ) ?
				!element.disabled :
				'a' === nodeName ?
					element.href || isTabIndexNotNaN :
					isTabIndexNotNaN
			);
	}

	// Focus inputs using cached focusable elements (see update())
	focusInputs(blurElems) {
		// Blurring body element in IE causes window.open windows to unfocus!
		if(focusableElems.length < 1 && blurElems.length) { blurElems.not('body').blur(); }

		// Focus the inputs
		else { focusableElems.first().focus(); }
	}

	// Steal focus from elements outside tooltip
	stealFocus(event) {
		if(!this.elem.is(':visible')) { return; }

		var target = $(event.target),
			tooltip = this.current.tooltip,
			container = target.closest(SELECTOR),
			targetOnTop;

		// Determine if input container target is above this
		targetOnTop = container.length < 1 ? false :
			(parseInt(container[0].style.zIndex, 10) > parseInt(tooltip[0].style.zIndex, 10));

		// If we're showing a modal, but focus has landed on an input below
		// this modal, divert focus to the first visible input in this modal
		// or if we can't find one... the tooltip itself
		if(!targetOnTop && target.closest(SELECTOR)[0] !== tooltip[0]) {
			this.focusInputs(target);
		}

		// Detect when we leave the last focusable element...
		this.onLast = event.target === focusableElems[focusableElems.length - 1];
	}
}
